import fedmsg.consumers
import datanommer.models

DEFAULTS = {
    'datanommer.enabled': True,
    'datanommer.sqlalchemy.url': 'sqlite:///datanommer.db',
}


import logging
log = logging.getLogger("fedmsg")


class Nommer(fedmsg.consumers.FedmsgConsumer):
    topic = "*"
    config_key = 'datanommer.enabled'

    def __init__(self, hub):
        super(Nommer, self).__init__(hub)

        # If fedmsg doesn't think we should be enabled, then we should quit
        # before setting up all the extra special zmq machinery.
        # __initialized is set in moksha.api.hub.consumer
        if not getattr(self, "__initialized", False):
            return

        # Setup a sqlalchemy DB connection (postgres, or sqlite)
        datanommer.models.init(self.hub.config['datanommer.sqlalchemy.url'])

    def consume(self, message):
        log.debug("Nomming %r" % message)
        datanommer.models.add(message['body'])



import copy
import os
import sqlalchemy.exc
import unittest

from nose.tools import raises
from nose.tools import eq_

import datanommer.models

filename = "datanommer-test.db"

scm_message = {
    "i": 1,
    "timestamp": 1344350850.8867381,
    "topic": "org.fedoraproject.prod.git.receive.valgrind.master",
    "msg": {
        "commit": {
            "stats": {
                "files": {
                    "valgrind.spec": {
                        "deletions": 2,
                        "lines": 3,
                        "insertions": 1
                    }
                },
                "total": {
                    "deletions": 2,
                    "files": 1,
                    "insertions": 1,
                    "lines": 3
                }
            },
            "name": "Mark Wielaard",
            "rev": "7a98f80d9b61ce167e4ef8129c81ed9284ecf4e1",
            "summary": "Clear CFLAGS CXXFLAGS LDFLAGS.",
            "message": """Clear CFLAGS CXXFLAGS LDFLAGS.
            This is a bit of a hammer.""",
            "email": "mjw@redhat.com",
            "branch": "master",
        }
    },
    "signature": "blah",
    "certificate": "blah",
}


class TestModels(unittest.TestCase):
    def setUp(self):
        datanommer.models.init("sqlite:///%s" % filename, create=True)

    def tearDown(self):
        os.remove(filename)

    @raises(KeyError)
    def test_add_empty(self):
        datanommer.models.add(dict())

    @raises(KeyError)
    def test_add_missing_i(self):
        msg = copy.deepcopy(scm_message)
        del msg['i']
        datanommer.models.add(msg)

    def test_add_missing_cert(self):
        msg = copy.deepcopy(scm_message)
        del msg['certificate']
        datanommer.models.add(msg)

    def test_add_nothing(self):
        eq_(datanommer.models.GitMessage.query.count(), 0)

    def test_add_and_check(self):
        msg = copy.deepcopy(scm_message)
        datanommer.models.add(msg)
        datanommer.models.session.flush()
        eq_(datanommer.models.GitMessage.query.count(), 1)



from sqlalchemy import create_engine
from sqlalchemy import (
    Column,
    DateTime,
    Integer,
    ForeignKey,
    Unicode,
)

from sqlalchemy.orm import sessionmaker, scoped_session
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.ext.hybrid import hybrid_property

import datetime
import fedmsg.encoding

maker = sessionmaker()
session = scoped_session(maker)

DeclarativeBase = declarative_base()
DeclarativeBase.query = session.query_property()

import logging
log = logging.getLogger("datanommer")


def init(uri, create=False):
    engine = create_engine(uri)
    session.configure(bind=engine)
    DeclarativeBase.query = session.query_property()

    if create:
        DeclarativeBase.metadata.create_all(engine)


def add(message):
    """ Take a dict-like fedmsg message and store it in the appropriate table.
    """

    possible = filter(lambda m: m.topic_filter in message['topic'], models)

    if len(possible) == 0:
        model_cls = UnclassifiedMessage
    elif len(possible) > 1:
        model_cls = possible[0]
        log.warn("Multiple models match message.  Using %r." % model)
    else:
        model_cls = possible[0]

    log.debug("Using %r" % model_cls)

    timestamp = message['timestamp']
    try:
        timestamp = datetime.datetime.fromtimestamp(timestamp)
    except Exception:
        pass

    obj = model_cls(
        i=message['i'],
        topic=message['topic'],
        timestamp=timestamp,
        certificate=message.get('certificate', None),
        signature=message.get('signature', None),
    )
    obj.msg = message['msg']

    session.add(obj)

    # TODO -- can we avoid committing every time?
    session.flush()
    session.commit()


class BaseMessage(object):
    id = Column(Integer, primary_key=True)
    i = Column(Integer, nullable=False)
    topic = Column(Unicode(255), nullable=False)
    timestamp = Column(DateTime, nullable=False)
    certificate = Column(Unicode(1023))
    signature = Column(Unicode(255))
    _msg = Column(Unicode(1023), nullable=False)

    @hybrid_property
    def msg(self):
        return fedmsg.encoding.loads(self._msg)

    @msg.setter
    def msg(self, dict_like_msg):
        self._msg = fedmsg.encoding.dumps(dict_like_msg)

    def __json__(self):
        return dict(
            i=self.i,
            topic=self.topic,
            timestamp=self.timestamp,
            certificate=self.certificate,
            signature=self.signature,
            msg=self.msg,
        )


class BodhiMessage(DeclarativeBase, BaseMessage):
    topic_filter = "bodhi"
    __tablename__ = "%s_messages" % topic_filter


class GitMessage(DeclarativeBase, BaseMessage):
    topic_filter = "git"
    __tablename__ = "%s_messages" % topic_filter


class WikiMessage(DeclarativeBase, BaseMessage):
    topic_filter = "wiki"
    __tablename__ = "%s_messages" % topic_filter


class TaggerMessage(DeclarativeBase, BaseMessage):
    topic_filter = "tagger"
    __tablename__ = "%s_messages" % topic_filter


class BusmonMessage(DeclarativeBase, BaseMessage):
    topic_filter = "busmon"
    __tablename__ = "%s_messages" % topic_filter


class FASMessage(DeclarativeBase, BaseMessage):
    topic_filter = "fas"
    __tablename__ = "%s_messages" % topic_filter


class MeetbotMessage(DeclarativeBase, BaseMessage):
    topic_filter = "meetbot"
    __tablename__ = "%s_messages" % topic_filter


class KojiMessage(DeclarativeBase, BaseMessage):
    topic_filter = "koji"
    __tablename__ = "%s_messages" % topic_filter


class LoggerMessage(DeclarativeBase, BaseMessage):
    topic_filter = "logger"
    __tablename__ = "%s_messages" % topic_filter


class HttpdMessage(DeclarativeBase, BaseMessage):
    topic_filter = "httpd"
    __tablename__ = "%s_messages" % topic_filter


class UnclassifiedMessage(DeclarativeBase, BaseMessage):
    topic_filter = "this will never be in a topic..."
    __tablename__ = "unclassified_messages"


models = frozenset((
    v for k, v in locals().items()
    if (
        isinstance(v, type) and
        issubclass(v, BaseMessage) and
        k is not "BaseMessage"
    )
))



from setuptools import setup, find_packages
import sys
import os

import multiprocessing
import logging

f = open('README.rst')
long_description = f.read().strip()
long_description = long_description.split('split here', 1)[1]
f.close()

version = '0.1.4'

setup(name='datanommer',
      version=version,
      description="A storage consumer for the Fedora Message Bus (fedmsg)",
      long_description=long_description,
      # Get strings from http://pypi.python.org/pypi?%3Aaction=list_classifiers
      classifiers=[],
      keywords='',
      author='Ralph Bean',
      author_email='rbean@redhat.com',
      url='http://github.com/ralphbean/datanommer',
      license='GPLv3+',
      packages=find_packages(exclude=['ez_setup', 'examples', 'tests']),
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          "fedmsg>=0.5.0",
          "sqlalchemy>=0.7",
      ],
      tests_require=[
          "nose",
      ],
      test_suite='nose.collector',
      entry_points={
          'console_scripts': (
              'datanommer-create-db=datanommer.commands:create',
              'datanommer-dump=datanommer.commands:dump',
              'datanommer-stats=datanommer.commands:stats',
          ),
          'moksha.consumer': (
              'noms = datanommer.consumer:Nommer'
          ),
      }
)



import datanommer.models
import fedmsg.encoding

from fedmsg.commands import command


@command(name="datanommer-create-db")
def create(**kw):
    """ Create a database and tables for 'datanommer.sqlalchemy.url' """
    datanommer.models.init(kw['datanommer.sqlalchemy.url'], create=True)


@command(name="datanommer-dump")
def dump(**kw):
    """ Dump the contents of the datanommer database as JSON """
    datanommer.models.init(kw['datanommer.sqlalchemy.url'])
    for model in datanommer.models.models:
        for entry in model.query.all():
            print fedmsg.encoding.pretty_dumps(entry)


@command(name="datanommer-stats")
def stats(**kw):
    """ Produce stats on the contents of the datanommer database """
    datanommer.models.init(kw['datanommer.sqlalchemy.url'])
    for model in datanommer.models.models:
        print model, "has", model.query.count(), "entries"
